/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.2.0
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2021, zouyaoji <370681295@qq.com>
 */
import _regeneratorRuntime from '@babel/runtime-corejs2/regenerator';
import _getIterator from '@babel/runtime-corejs2/core-js/get-iterator';
import cmp from '../virtualCmp';
import './icon-move';
import './icon-add';
import './icon-delete';
import _parseFloat from '@babel/runtime-corejs2/core-js/parse-float';

//
//
//
//
var icons = {};
var notLoadedIcons = [];
var defaultWidth = '';
var defaultHeight = '';
var classPrefix = 'vc-svg';
var isStroke = false;
var isOriginalDefault = false;
var script = {
  name: 'svgicon',
  data: function data() {
    return {
      loaded: false
    };
  },
  props: {
    icon: String,
    name: String,
    width: {
      type: String,
      default: ''
    },
    height: {
      type: String,
      default: ''
    },
    scale: String,
    dir: String,
    fill: {
      type: Boolean,
      default: function _default() {
        return !isStroke;
      }
    },
    color: String,
    original: {
      type: Boolean,
      default: function _default() {
        return isOriginalDefault;
      }
    },
    title: String
  },
  computed: {
    clazz: function clazz() {
      var clazz = "".concat(classPrefix, "-icon");

      if (this.fill) {
        clazz += " ".concat(classPrefix, "-fill");
      }

      if (this.dir) {
        clazz += " ".concat(classPrefix, "-").concat(this.dir);
      }

      return clazz;
    },
    iconName: function iconName() {
      return this.name || this.icon;
    },
    iconData: function iconData() {
      var iconData = icons[this.iconName];

      if (iconData || this.loaded) {
        return iconData;
      }

      return null;
    },
    colors: function colors() {
      if (this.color) {
        return this.color.split(' ');
      }

      return [];
    },
    path: function path() {
      var pathData = '';

      if (this.iconData) {
        pathData = this.iconData.data;
        pathData = this.setTitle(pathData); // use original color

        if (this.original) {
          pathData = this.addOriginalColor(pathData);
        }

        if (this.colors.length > 0) {
          pathData = this.addColor(pathData);
        }
      } else {
        // if no iconData, push to notLoadedIcons
        notLoadedIcons.push({
          name: this.iconName,
          component: this
        });
      }

      return this.getValidPathData(pathData);
    },
    box: function box() {
      var width = this.width || 16;
      var height = this.width || 16;

      if (this.iconData) {
        if (this.iconData.viewBox) {
          return this.iconData.viewBox;
        }

        return "0 0 ".concat(this.iconData.width, " ").concat(this.iconData.height);
      }

      return "0 0 ".concat(_parseFloat(width), " ").concat(_parseFloat(height));
    },
    style: function style() {
      var digitReg = /^\d+$/;
      var scale = Number(this.scale);
      var width;
      var height; // apply scale

      if (!isNaN(scale) && this.iconData) {
        width = Number(this.iconData.width) * scale + 'px';
        height = Number(this.iconData.height) * scale + 'px';
      } else {
        width = digitReg.test(this.width) ? this.width + 'px' : this.width || defaultWidth;
        height = digitReg.test(this.height) ? this.height + 'px' : this.height || defaultHeight;
      }

      var style = {};

      if (width) {
        style.width = width;
      }

      if (height) {
        style.height = height;
      }

      return style;
    }
  },
  created: function created() {
    if (icons[this.iconName]) {
      this.loaded = true;
    }
  },
  methods: {
    addColor: function addColor(data) {
      var _this = this;

      var reg = /<(path|rect|circle|polygon|line|polyline|ellipse)\s/gi;
      var i = 0;
      return data.replace(reg, function (match) {
        var color = _this.colors[i++] || _this.colors[_this.colors.length - 1];
        var fill = _this.fill; // if color is '_', ignore it

        if (color && color === '_') {
          return match;
        } // if color start with 'r-', reverse the fill value


        if (color && color.indexOf('r-') === 0) {
          fill = !fill;
          color = color.split('r-')[1];
        }

        var style = fill ? 'fill' : 'stroke';
        var reverseStyle = fill ? 'stroke' : 'fill';
        return match + "".concat(style, "=\"").concat(color, "\" ").concat(reverseStyle, "=\"none\" ");
      });
    },
    addOriginalColor: function addOriginalColor(data) {
      var styleReg = /_fill="|_stroke="/gi;
      return data.replace(styleReg, function (styleName) {
        return styleName && styleName.slice(1);
      });
    },
    getValidPathData: function getValidPathData(pathData) {
      // If use original and colors, clear double fill or stroke
      if (this.original && this.colors.length > 0) {
        // eslint-disable-next-line no-useless-escape
        var reg = /<(path|rect|circle|polygon|line|polyline|ellipse)(\sfill|\sstroke)([="\w\s\.\-\+#\$\&>]+)(fill|stroke)/gi;
        pathData = pathData.replace(reg, function (match, p1, p2, p3, p4) {
          return "<".concat(p1).concat(p2).concat(p3, "_").concat(p4);
        });
      }

      return pathData;
    },
    setTitle: function setTitle(pathData) {
      if (this.title) {
        var title = this.title // eslint-disable-next-line no-useless-escape
        .replace(/\</gi, '&lt;').replace(/>/gi, '&gt;').replace(/&/g, '&amp;');
        return "<title>".concat(title, "</title>") + pathData;
      }

      return pathData;
    },
    onClick: function onClick(e) {
      this.$emit('click', e);
    }
  },
  install: function install(Vue) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var tagName = options.tagName || 'svgicon';

    if (options.classPrefix) {
      classPrefix = options.classPrefix;
    }

    isStroke = !!options.isStroke;
    isOriginalDefault = !!options.isOriginalDefault; // default size

    options.defaultWidth && (defaultWidth = options.defaultWidth);
    options.defaultHeight && (defaultHeight = options.defaultHeight);
    Vue.component(tagName, this);
  },
  // register icons
  register: function register(data) {
    var _loop = function _loop(name) {
      if (!icons[name]) {
        icons[name] = data[name];
      } // check new register icon is not loaded, and set loaded to true


      notLoadedIcons = notLoadedIcons.filter(function (v, ix) {
        if (v.name === name) {
          v.component.$set(v.component, 'loaded', true);
        }

        return v.name !== name;
      });
    };

    for (var name in data) {
      _loop(name);
    }
  },
  icons: icons
};

/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('svg', {
    class: _vm.clazz,
    style: _vm.style,
    attrs: {
      "viewBox": _vm.box,
      "version": "1.1"
    },
    domProps: {
      "innerHTML": _vm._s(_vm.path)
    },
    on: {
      "click": _vm.onClick
    }
  });
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = "data-v-b5224d78";
/* functional template */

var __vue_is_functional_template__ = false;
/* component normalizer */

function __vue_normalize__(template, style, script, scope, functional, moduleIdentifier, createInjector, createInjectorSSR) {
  var component = (typeof script === 'function' ? script.options : script) || {}; // For security concerns, we use only base name in production mode.

  component.__file = "VcIconSvg.vue";

  if (!component.render) {
    component.render = template.render;
    component.staticRenderFns = template.staticRenderFns;
    component._compiled = true;
    if (functional) component.functional = true;
  }

  component._scopeId = scope;

  return component;
}
/* style inject */

/* style inject SSR */


var VcIconSvg = __vue_normalize__({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__);

var props = {
  mode: {
    type: Number,
    default: 1
  },
  show: {
    type: Boolean,
    default: true
  },
  pointColor: {
    type: String | Object | Array,
    default: 'rgb(255,229,0)'
  },
  pointPixelSize: {
    type: Number,
    default: 8
  },
  editable: {
    type: Boolean,
    default: false
  },
  showDrawTip: {
    type: Boolean,
    default: true
  }
};
var watch = {
  editable: function editable(val) {
    if (!val) {
      this.showToolbar = false;
    } else {
      this.drawing = false;
    }
  },
  drawing: function drawing(val) {
    var nextTick, polylines, startNew, drawType, $parent, getParent, polyline, drawCmpNames, measureCmpNames, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, $node, listener;

    return _regeneratorRuntime.async(function drawing$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            nextTick = false;
            polylines = this.polylines, startNew = this.startNew, drawType = this.drawType, $parent = this.$parent, getParent = this.getParent;
            polyline = polylines[polylines.length - 1];

            if (!(!val && polyline && !polyline.positions.length)) {
              _context.next = 8;
              break;
            }

            this.polylines.pop();
            this.showTooltip = false;
            _context.next = 47;
            break;

          case 8:
            if (!val) {
              _context.next = 46;
              break;
            }

            drawCmpNames = [];
            _context.t0 = drawType;
            _context.next = _context.t0 === 'pointDrawing' ? 13 : _context.t0 === 'polylineDrawing' ? 16 : _context.t0 === 'polygonDrawing' ? 19 : 22;
            break;

          case 13:
            drawCmpNames.push('vc-handler-draw-polyline');
            drawCmpNames.push('vc-handler-draw-polygon');
            return _context.abrupt("break", 22);

          case 16:
            drawCmpNames.push('vc-handler-draw-point');
            drawCmpNames.push('vc-handler-draw-polygon');
            return _context.abrupt("break", 22);

          case 19:
            drawCmpNames.push('vc-handler-draw-polyline');
            drawCmpNames.push('vc-handler-draw-point');
            return _context.abrupt("break", 22);

          case 22:
            measureCmpNames = ['vc-measure-height', 'vc-measure-distance', 'vc-measure-area'];
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context.prev = 26;

            for (_iterator = _getIterator(getParent($parent).$slots.default || []); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              $node = _step.value;

              if ($node.componentOptions && drawCmpNames.indexOf($node.componentOptions.tag) !== -1) {
                $node.child.drawing = false;
                nextTick = true;
              }

              if ($node.componentOptions && measureCmpNames.indexOf($node.componentOptions.tag) !== -1) {
                $node.child.measuring = false;
                nextTick = true;
              }
            }

            _context.next = 34;
            break;

          case 30:
            _context.prev = 30;
            _context.t1 = _context["catch"](26);
            _didIteratorError = true;
            _iteratorError = _context.t1;

          case 34:
            _context.prev = 34;
            _context.prev = 35;

            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }

          case 37:
            _context.prev = 37;

            if (!_didIteratorError) {
              _context.next = 40;
              break;
            }

            throw _iteratorError;

          case 40:
            return _context.finish(37);

          case 41:
            return _context.finish(34);

          case 42:
            startNew();
            this.showTooltip = true;
            _context.next = 47;
            break;

          case 46:
            this.showTooltip = false;

          case 47:
            if (!this.showTooltip) {
              this.tooltipPosition = [0, 0, 0];
            }

            _context.t2 = nextTick;

            if (!_context.t2) {
              _context.next = 52;
              break;
            }

            _context.next = 52;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 52:
            this.viewer.canvas.setAttribute('style', val ? 'cursor: crosshair' : 'cursor: auto');
            listener = this.$listeners.activeEvt;
            listener && this.$emit('activeEvt', {
              type: drawType,
              isActive: val
            });

          case 55:
          case "end":
            return _context.stop();
        }
      }
    }, null, this, [[26, 30, 34, 42], [35,, 37, 41]]);
  }
};
var computed = {
  points: function points() {
    var _this = this;

    var points = [];
    this.polylines.forEach(function (polyline, index) {
      polyline.positions.forEach(function (position, subIndex) {
        var point = {
          color: _this.pointColor,
          pixelSize: _this.pointPixelSize,
          position: position,
          polylineIndex: index,
          positionIndex: subIndex
        };
        points.push(point);
      });
    });
    return points;
  }
};
var methods = {
  createCesiumObject: function createCesiumObject() {
    var viewer, handler;
    return _regeneratorRuntime.async(function createCesiumObject$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            viewer = this.viewer;
            handler = new Cesium.ScreenSpaceEventHandler(viewer.canvas);
            handler.setInputAction(this.LEFT_CLICK, Cesium.ScreenSpaceEventType.LEFT_CLICK);
            handler.setInputAction(this.MOUSE_MOVE, Cesium.ScreenSpaceEventType.MOUSE_MOVE);
            handler.setInputAction(this.RIGHT_CLICK, Cesium.ScreenSpaceEventType.RIGHT_CLICK);
            this.handler = handler;
            return _context2.abrupt("return", this.polylines);

          case 7:
          case "end":
            return _context2.stop();
        }
      }
    }, null, this);
  },
  mount: function mount() {
    return _regeneratorRuntime.async(function mount$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            return _context3.abrupt("return", true);

          case 1:
          case "end":
            return _context3.stop();
        }
      }
    });
  },
  unload: function unload() {
    return _regeneratorRuntime.async(function unload$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            return _context4.abrupt("return", true);

          case 1:
          case "end":
            return _context4.stop();
        }
      }
    });
  },
  LEFT_CLICK: function LEFT_CLICK(movement) {
    if (!this.drawing) {
      return;
    }

    if (this.editingPoint) {
      this.RIGHT_CLICK(movement);
      return;
    }

    var Cesium = this.Cesium,
        viewer = this.viewer,
        polylines = this.polylines,
        onDrawingEvt = this.onDrawingEvt;
    var cartesian = viewer.scene.pickPosition(movement.position);

    if (!Cesium.defined(cartesian)) {
      return;
    }

    var nIndex = polylines.length - 1;
    var polyline = polylines[nIndex];
    polyline.positions.push(cartesian);
    onDrawingEvt(polyline, nIndex);
  },
  MOUSE_MOVE: function MOUSE_MOVE(movement) {
    var Cesium, viewer, polylines, onDrawingEvt, drawType, cartesian, nIndex, polyline, listener;
    return _regeneratorRuntime.async(function MOUSE_MOVE$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            if (this.drawing) {
              _context5.next = 2;
              break;
            }

            return _context5.abrupt("return");

          case 2:
            Cesium = this.Cesium, viewer = this.viewer, polylines = this.polylines, onDrawingEvt = this.onDrawingEvt, drawType = this.drawType;
            cartesian = viewer.scene.pickPosition(movement.endPosition);

            if (Cesium.defined(cartesian)) {
              _context5.next = 6;
              break;
            }

            return _context5.abrupt("return");

          case 6:
            this.tooltipPosition = cartesian;
            this.tooltip = this.$vc.lang.draw.drawingTip1;

            if (polylines.length) {
              _context5.next = 10;
              break;
            }

            return _context5.abrupt("return");

          case 10:
            nIndex = this.editingPoint ? this.editingPoint.polylineIndex : polylines.length - 1;
            polyline = polylines[nIndex];

            if (polyline.positions.length) {
              _context5.next = 14;
              break;
            }

            return _context5.abrupt("return");

          case 14:
            if (polyline.positions.length >= 2) {
              this.tooltip = this.editingPoint ? this.$vc.lang.draw.drawingTip3 : this.$vc.lang.draw.drawingTip2;
            }

            if (this.editingPoint) {
              polyline.positions.splice(this.editingPoint.positionIndex, 1, cartesian);
            } else {
              if (polyline.positions.length >= 2) {
                polyline.positions.pop();
              }

              polyline.positions.push(cartesian);
            }

            listener = this.$listeners.movingEvt;
            listener && this.$emit('movingEvt', movement.endPosition, drawType);
            onDrawingEvt(polyline, nIndex);

          case 19:
          case "end":
            return _context5.stop();
        }
      }
    }, null, this);
  },
  RIGHT_CLICK: function RIGHT_CLICK(movement) {
    var viewer, polylines, mode, startNew, onDrawingEvt, nIndex, polyline, cartesian;
    return _regeneratorRuntime.async(function RIGHT_CLICK$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            if (this.drawing) {
              _context6.next = 2;
              break;
            }

            return _context6.abrupt("return");

          case 2:
            viewer = this.viewer, polylines = this.polylines, mode = this.mode, startNew = this.startNew, onDrawingEvt = this.onDrawingEvt;

            if (polylines.length) {
              _context6.next = 5;
              break;
            }

            return _context6.abrupt("return");

          case 5:
            nIndex = this.editingPoint ? this.editingPoint.polylineIndex : polylines.length - 1; // const nIndex = polylines.length - 1

            polyline = polylines[nIndex];

            if (!(polyline.positions.length === 0)) {
              _context6.next = 9;
              break;
            }

            return _context6.abrupt("return");

          case 9:
            cartesian = viewer.scene.pickPosition(movement.position);

            if (Cesium.defined(cartesian)) {
              _context6.next = 12;
              break;
            }

            return _context6.abrupt("return");

          case 12:
            if (!this.editingPoint) {
              if (polyline.positions.length > 1) {
                polyline.positions.pop();
              }

              if (mode === 0) {
                startNew();
              } else {
                this.drawing = false;
              }
            } else {
              this.editingPoint = undefined;
              this.drawing = false;
            }

            _context6.next = 15;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 15:
            onDrawingEvt(polyline, nIndex, true);

          case 16:
          case "end":
            return _context6.stop();
        }
      }
    }, null, this);
  },
  startNew: function startNew() {
    var polylines = this.polylines;
    var poyline = {
      positions: []
    };
    Cesium.defined(polylines) && polylines.push(poyline);
  },
  clear: function clear() {
    this.polylines = [];
    this.drawing = false;
  },
  onDrawingEvt: function onDrawingEvt(polyline, index) {
    var flag = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    this.index = index;

    if (!this.depthTest) {
      var rs = Cesium.RenderState.fromCache({
        depthMask: true,
        depthTest: {
          enabled: false
        }
      });
      this.$refs.pointCollection && (this.$refs.pointCollection.cesiumObject._rsOpaque = rs);

      if (Cesium.SuperMapImageryProvider) {
        this.$refs.polylineCollection && (this.$refs.polylineCollection.cesiumObject._opaqueRS = rs);
      } else {
        this.$refs.polylineCollection && (this.$refs.polylineCollection.cesiumObject._opaqueRS.depthTest.enabled = false);
      }
    }

    var listener = this.$listeners.drawEvt;
    listener && this.$emit('drawEvt', {
      polyline: polyline,
      type: this.drawType,
      finished: flag
    });
  },
  pointMouseOver: function pointMouseOver(e) {
    if (!this.editable) {
      return;
    }

    if (this.editingPoint) {
      return;
    }

    e.pickedFeature.primitive.pixelSize = this.pointPixelSize * 2.0;
    this.toolbarPosition = e.pickedFeature.primitive.position;
    this.showToolbar = true;
    this.mouseoverPoint = e.pickedFeature.primitive;
  },
  pointMouseOut: function pointMouseOut(e) {
    if (!this.editable) {
      return;
    }

    e.pickedFeature.primitive.pixelSize = this.pointPixelSize * 1.0;
    this.toolbarPosition = [0, 0, 0];
    this.showToolbar = false;
    this.mouseoverPoint = undefined;
  },
  onEditClick: function onEditClick(e) {
    this.editType = e;
    this.toolbarPosition = [0, 0, 0];
    this.showToolbar = false;

    if (!this.editable) {
      return;
    }

    this.drawing = false;

    if (e === 'delete') {
      var nIndex = this.mouseoverPoint.polylineIndex;
      var polyline = this.polylines[nIndex];
      polyline.positions.splice(this.mouseoverPoint.positionIndex, 1);
    } else if (e === 'insert') {
      var _nIndex = this.mouseoverPoint.polylineIndex;
      var _polyline = this.polylines[_nIndex];

      _polyline.positions.splice(this.mouseoverPoint.positionIndex, 0, this.mouseoverPoint.position);

      this.editingPoint = this.mouseoverPoint;
      this.drawing = true;
    } else {
      this.editingPoint = this.mouseoverPoint;
      this.drawing = true;
    }
  }
};
var mixinDraw = {
  mixins: [cmp],
  data: function data() {
    return {
      drawing: false,
      polylines: [],
      toolbarPosition: [0, 0, 0],
      showToolbar: false,
      tooltipPosition: [0, 0, 0],
      showTooltip: false,
      tooltip: '',
      nowaiting: true
    };
  },
  components: {
    VcIconSvg: VcIconSvg
  },
  props: props,
  watch: watch,
  computed: computed,
  methods: methods,
  destroyed: function destroyed() {
    var handler = this.handler;

    if (handler) {
      handler.removeInputAction(Cesium.ScreenSpaceEventType.LEFT_CLICK);
      handler.removeInputAction(Cesium.ScreenSpaceEventType.MOUSE_MOVE);
      handler.removeInputAction(Cesium.ScreenSpaceEventType.RIGHT_CLICK);
      handler.destroy();
    }
  }
};

export default mixinDraw;
