/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.2.0
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2021, zouyaoji <370681295@qq.com>
 */
import _Object$defineProperties from '@babel/runtime-corejs2/core-js/object/define-properties';
import _Object$assign from '@babel/runtime-corejs2/core-js/object/assign';
import _regeneratorRuntime from '@babel/runtime-corejs2/regenerator';
import _getIterator from '@babel/runtime-corejs2/core-js/get-iterator';
import cmp from '../virtualCmp';

var props = {
  mode: {
    type: Number,
    default: 1
  },
  font: {
    type: String,
    default: '100 20px SimSun'
  },
  fillColor: {
    type: String | Object | Array,
    default: 'WHITE'
  },
  labelStyle: {
    type: Number,
    default: 2
  },
  showBackground: {
    type: Boolean,
    default: true
  },
  backgroundColor: {
    type: String | Object | Array,
    default: 'rgba(38, 38, 38, 0.85)'
  },
  outlineWidth: {
    type: Number,
    default: 1
  },
  outlineColor: {
    type: String | Object | Array,
    default: 'BLUE'
  },
  pixelOffset: {
    type: Object,
    default: function _default() {
      return {
        x: 15,
        y: -20
      };
    }
  },
  pointColor: {
    type: String | Object | Array,
    default: 'rgb(255,229,0)'
  },
  pointPixelSize: {
    type: Number,
    default: 8
  },
  polylineMaterial: {
    type: Object,
    default: function _default() {
      return {
        fabric: {
          type: 'Color',
          uniforms: {
            color: '#51ff00'
          }
        }
      };
    }
  },
  polylineWidth: {
    type: Number,
    default: 2
  },
  depthTest: {
    type: Boolean,
    default: false
  }
};
var watch = {
  measuring: function measuring(val) {
    var nextTick, polylines, startNew, type, $parent, getParent, polyline, measureCmpNames, drawCmpNames, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, $node, listener;

    return _regeneratorRuntime.async(function measuring$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            nextTick = false;
            polylines = this.polylines, startNew = this.startNew, type = this.type, $parent = this.$parent, getParent = this.getParent;
            polyline = polylines[polylines.length - 1];

            if (!(!val && polyline && !polyline.positions.length)) {
              _context.next = 7;
              break;
            }

            this.polylines.pop();
            _context.next = 42;
            break;

          case 7:
            if (!val) {
              _context.next = 42;
              break;
            }

            measureCmpNames = [];
            _context.t0 = type;
            _context.next = _context.t0 === 'distanceMeasuring' ? 12 : _context.t0 === 'areaMeasuring' ? 15 : _context.t0 === 'heightMeasuring' ? 18 : 21;
            break;

          case 12:
            measureCmpNames.push('vc-measure-height');
            measureCmpNames.push('vc-measure-area');
            return _context.abrupt("break", 21);

          case 15:
            measureCmpNames.push('vc-measure-distance');
            measureCmpNames.push('vc-measure-height');
            return _context.abrupt("break", 21);

          case 18:
            measureCmpNames.push('vc-measure-distance');
            measureCmpNames.push('vc-measure-area');
            return _context.abrupt("break", 21);

          case 21:
            drawCmpNames = ['vc-handler-draw-polyline', 'vc-handler-draw-point', 'vc-handler-draw-polygon'];
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context.prev = 25;

            for (_iterator = _getIterator(getParent($parent).$slots.default || []); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              $node = _step.value;

              if ($node.componentOptions && measureCmpNames.indexOf($node.componentOptions.tag) !== -1) {
                $node.child.measuring = false;
                nextTick = true;
              }

              if ($node.componentOptions && drawCmpNames.indexOf($node.componentOptions.tag) !== -1) {
                $node.child.drawing = false;
                nextTick = true;
              }
            }

            _context.next = 33;
            break;

          case 29:
            _context.prev = 29;
            _context.t1 = _context["catch"](25);
            _didIteratorError = true;
            _iteratorError = _context.t1;

          case 33:
            _context.prev = 33;
            _context.prev = 34;

            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }

          case 36:
            _context.prev = 36;

            if (!_didIteratorError) {
              _context.next = 39;
              break;
            }

            throw _iteratorError;

          case 39:
            return _context.finish(36);

          case 40:
            return _context.finish(33);

          case 41:
            startNew();

          case 42:
            _context.t2 = nextTick;

            if (!_context.t2) {
              _context.next = 46;
              break;
            }

            _context.next = 46;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 46:
            this.viewer.canvas.setAttribute('style', val ? 'cursor: crosshair' : 'cursor: auto');
            listener = this.$listeners.activeEvt;
            listener && this.$emit('activeEvt', {
              type: type,
              isActive: val
            });

          case 49:
          case "end":
            return _context.stop();
        }
      }
    }, null, this, [[25, 29, 33, 41], [34,, 36, 40]]);
  }
};
var methods = {
  createCesiumObject: function createCesiumObject() {
    var viewer, handler;
    return _regeneratorRuntime.async(function createCesiumObject$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            viewer = this.viewer;
            viewer.scene.frameState.morphTime = 0;
            handler = new Cesium.ScreenSpaceEventHandler(viewer.canvas);
            handler.setInputAction(this.LEFT_CLICK, Cesium.ScreenSpaceEventType.LEFT_CLICK);
            handler.setInputAction(this.MOUSE_MOVE, Cesium.ScreenSpaceEventType.MOUSE_MOVE);
            handler.setInputAction(this.RIGHT_CLICK, Cesium.ScreenSpaceEventType.RIGHT_CLICK);
            this.handler = handler;
            this.enterMoveAction = false;
            this.lastCartesianRemoved = false;
            return _context2.abrupt("return", this.polylines);

          case 10:
          case "end":
            return _context2.stop();
        }
      }
    }, null, this);
  },
  mount: function mount() {
    return _regeneratorRuntime.async(function mount$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            return _context3.abrupt("return", true);

          case 1:
          case "end":
            return _context3.stop();
        }
      }
    });
  },
  unmount: function unmount() {
    return _regeneratorRuntime.async(function unmount$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            return _context4.abrupt("return", true);

          case 1:
          case "end":
            return _context4.stop();
        }
      }
    });
  },
  LEFT_CLICK: function LEFT_CLICK(movement) {
    if (!this.measuring) {
      return;
    }

    var Cesium = this.Cesium,
        viewer = this.viewer,
        polylines = this.polylines,
        type = this.type,
        onMeasureEvt = this.onMeasureEvt;
    var cartesian = viewer.scene.pickPosition(movement.position);

    if (!Cesium.defined(cartesian)) {
      return;
    }

    var nIndex = polylines.length - 1;
    var polyline = polylines[nIndex];

    if (type === 'distanceMeasuring' || type === 'areaMeasuring') {
      // 鼠标移动事件添加的移动点需要移除
      if (this.enterMoveAction) {
        polyline.positions.pop();
        polyline.distances && polyline.distances.pop();
        this.lastCartesianRemoved = true;
      }
    }

    polyline.positions.push(cartesian);
    var length = polyline.positions.length;

    if (length > 1) {
      // 重复点不计算
      if (Cesium.Cartesian3.distance(polyline.positions[length - 1], polyline.positions[length - 2]) < 1e-3) {
        polyline.positions.pop();
        return;
      }
    }

    switch (type) {
      case 'distanceMeasuring':
        polyline.distance = this.getDistance(polyline.positions);
        polyline.distances.push(polyline.distance);
        onMeasureEvt(polyline, nIndex);
        break;

      case 'areaMeasuring':
        polyline.area = this.getSurfaceArea(polyline.positions);
        polyline.projectedArea = this.getProjectedArea(polyline.positions);
        polyline.distance = this.getDistance(polyline.positions);
        polyline.distances.push(polyline.distance);

        if (polyline.positions.length > 2 && !this.enterMoveAction) {
          if (this.moveLastDistance) {
            polyline.distances.pop();
          }

          var clonePoistions = Cesium.clone(polyline.positions, true);
          clonePoistions.push(polyline.positions[0]);
          var distance = this.getDistance(clonePoistions);
          polyline.distances.push(distance);
          polyline.distance = distance;
          this.moveLastDistance = true;
        }

        onMeasureEvt(polyline, nIndex);
        break;

      case 'heightMeasuring':
        polyline.positions.pop();

        if (polyline.positions.length === 0) {
          polyline.positions.push(cartesian);
          this.startPoint = cartesian;
        }

        onMeasureEvt(polyline, this.labels);
        break;
    }
  },
  MOUSE_MOVE: function MOUSE_MOVE(movement) {
    var viewer, polylines, onMeasureEvt, type, nIndex, polyline, cartesian, listener, distance, _distance, clonePoistions, labels;

    return _regeneratorRuntime.async(function MOUSE_MOVE$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            if (this.measuring) {
              _context5.next = 2;
              break;
            }

            return _context5.abrupt("return");

          case 2:
            viewer = this.viewer, polylines = this.polylines, onMeasureEvt = this.onMeasureEvt, type = this.type;

            if (polylines.length) {
              _context5.next = 5;
              break;
            }

            return _context5.abrupt("return");

          case 5:
            nIndex = polylines.length - 1;
            polyline = polylines[nIndex];

            if (polyline.positions.length) {
              _context5.next = 9;
              break;
            }

            return _context5.abrupt("return");

          case 9:
            cartesian = viewer.scene.pickPosition(movement.endPosition);

            if (Cesium.defined(cartesian)) {
              _context5.next = 12;
              break;
            }

            return _context5.abrupt("return");

          case 12:
            this.enterMoveAction = true;
            listener = this.$listeners.movingEvt;
            listener && this.$emit('movingEvt', movement.endPosition, type);

            if (!(type === 'distanceMeasuring' || type === 'areaMeasuring')) {
              _context5.next = 25;
              break;
            }

            if (polyline.positions.length >= 2) {
              // 如果鼠标左键点击事件移除了 不需要再移除
              if (!this.lastCartesianRemoved) {
                polyline.positions.pop();
                polyline.distances && polyline.distances.pop();
              }

              if (type === 'areaMeasuring') {
                polyline.distances && polyline.distances.pop();
              }
            }

            polyline.positions.push(cartesian);
            this.lastCartesianRemoved = false;

            if (type === 'distanceMeasuring') {
              distance = this.getDistance(polyline.positions);
              polyline.distances.push(distance);
              polyline.distance = distance;
              nIndex = polylines.reduce(function (pre, cur) {
                return pre + cur.positions.length - 1;
              }, 0) - 1;
            } else {
              polyline.area = this.getSurfaceArea(polyline.positions);
              polyline.projectedArea = this.getProjectedArea(polyline.positions);
              _distance = this.getDistance(polyline.positions);
              polyline.distances.push(_distance);
              polyline.distance = _distance;

              if (polyline.positions.length >= 2) {
                clonePoistions = Cesium.clone(polyline.positions, true);
                clonePoistions.push(polyline.positions[0]);
                _distance = this.getDistance(clonePoistions);
                polyline.distances.push(_distance);
                polyline.distance = _distance;
              }
            }

            _context5.next = 22;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 22:
            onMeasureEvt(polyline, nIndex);
            _context5.next = 30;
            break;

          case 25:
            labels = this.labels;
            this.getHeight(cartesian, polyline);
            _context5.next = 29;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 29:
            onMeasureEvt(polyline, labels);

          case 30:
          case "end":
            return _context5.stop();
        }
      }
    }, null, this);
  },
  RIGHT_CLICK: function RIGHT_CLICK(movement) {
    var viewer, polylines, mode, startNew, onMeasureEvt, type, removeLastPosition, nIndex, polyline, cartesian, getDistance, getSurfaceArea, getProjectedArea, clonePoistions, distance, labels;
    return _regeneratorRuntime.async(function RIGHT_CLICK$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            if (this.measuring) {
              _context6.next = 2;
              break;
            }

            return _context6.abrupt("return");

          case 2:
            viewer = this.viewer, polylines = this.polylines, mode = this.mode, startNew = this.startNew, onMeasureEvt = this.onMeasureEvt, type = this.type, removeLastPosition = this.removeLastPosition;

            if (polylines.length) {
              _context6.next = 5;
              break;
            }

            return _context6.abrupt("return");

          case 5:
            nIndex = polylines.length - 1;
            polyline = polylines[nIndex];

            if (!(polyline.positions.length === 0)) {
              _context6.next = 9;
              break;
            }

            return _context6.abrupt("return");

          case 9:
            cartesian = viewer.scene.pickPosition(movement.position);

            if (Cesium.defined(cartesian)) {
              _context6.next = 12;
              break;
            }

            return _context6.abrupt("return");

          case 12:
            if (type === 'distanceMeasuring') {
              if (removeLastPosition) {
                polyline.positions.pop();
                polyline.distances.pop();
              }

              getDistance = this.getDistance;
              polyline.distance = getDistance(polyline.positions);

              if (polyline.positions.length === 1) {
                polyline.positions = [];
              }

              nIndex = polylines.reduce(function (pre, cur) {
                return pre + cur.positions.length - 1;
              }, 0) - 1;
            } else if (type === 'areaMeasuring') {
              if (removeLastPosition) {
                polyline.positions.pop();
                polyline.distances.pop();
              }

              getSurfaceArea = this.getSurfaceArea, getProjectedArea = this.getProjectedArea;
              polyline.area = getSurfaceArea(polyline.positions);
              polyline.projectedArea = getProjectedArea(polyline.positions);
              polyline.distance = this.getDistance(polyline.positions);

              if (polyline.positions.length >= 2 && this.enterMoveAction) {
                polyline.distances.pop();
                clonePoistions = Cesium.clone(polyline.positions, true);
                clonePoistions.push(polyline.positions[0]);
                distance = this.getDistance(clonePoistions);
                polyline.distances.push(distance);
                polyline.distance = distance;
              }

              this.moveLastDistance = false;

              if (polyline.positions.length <= 2) {
                polyline.positions = [];
              }
            } else {
              this.getHeight(cartesian, polyline);
            }

            if (mode === 0) {
              startNew();
            } else {
              this.measuring = false;
            }

            _context6.next = 16;
            return _regeneratorRuntime.awrap(this.$nextTick());

          case 16:
            if (type === 'distanceMeasuring' || type === 'areaMeasuring') {
              onMeasureEvt(polyline, nIndex, true);
            } else {
              labels = this.labels;
              onMeasureEvt(polyline, labels, true);
            }

          case 17:
          case "end":
            return _context6.stop();
        }
      }
    }, null, this);
  },
  startNew: function startNew() {
    var polylines = this.polylines,
        type = this.type;

    if (!Cesium.defined(polylines)) {
      return;
    }

    var polyline = {
      positions: []
    };

    if (type === 'distanceMeasuring') {
      _Object$assign(polyline, {
        distances: [],
        distance: 0
      });
    } else if (type === 'areaMeasuring') {
      _Object$assign(polyline, {
        area: 0,
        projectedArea: 0,
        distances: [],
        distance: 0
      });
    } else {
      _Object$assign(polyline, {
        distanceH: 0,
        height: 0,
        distanceS: 0
      });
    }

    polylines.push(polyline);
  },

  /**
   * 根据传入坐标数组计算距离。
   * @param {Array.Cartesian3} positions 传入的坐标数组
   * @returns {Number} 返回长度数值。
   */
  getDistance: function getDistance(positions) {
    var _Cesium = Cesium,
        Cartesian3 = _Cesium.Cartesian3;
    var clampToGround = this.clampToGround,
        getGeodesicDistance = this.getGeodesicDistance;
    var distance = 0;

    for (var i = 0; i < positions.length - 1; i++) {
      var s = 0;

      if (clampToGround) {
        // Cartesian.distance gives the straight line distance between the two points, ignoring curvature. This is not what we want.
        // Cartesian3.distance 计算的是两点之间的直线距离，忽略了地球曲率，贴地时不太合理。
        // 2.0.3 版本增加测地线距离（GeodesicDistance）。
        s = getGeodesicDistance(positions[i], positions[i + 1]);
      } else {
        s = Cartesian3.distance(positions[i], positions[i + 1]);
      }

      distance = distance + s;
    }

    return distance;
  },

  /**
   * 返回两点之间的测地距离。
   * @param {Cartesian3} pointOne 第一个坐标点
   * @param {Cartesian3} pointTwo 第二个坐标点
   * @returns {Number} 返回两点之间的测地距离。
   */
  getGeodesicDistance: function getGeodesicDistance(pointOne, pointTwo) {
    var _Cesium2 = Cesium,
        Ellipsoid = _Cesium2.Ellipsoid,
        EllipsoidGeodesic = _Cesium2.EllipsoidGeodesic;
    var pickedPointCartographic = Ellipsoid.WGS84.cartesianToCartographic(pointOne);
    var lastPointCartographic = Ellipsoid.WGS84.cartesianToCartographic(pointTwo);
    var geodesic = new EllipsoidGeodesic(pickedPointCartographic, lastPointCartographic);
    return geodesic.surfaceDistance;
  },
  getHeight: function getHeight(endPoint, polyline) {
    var labels = this.labels; // let endPoint = cartesian

    var normalStart = {};
    Cesium.Cartesian3.normalize(this.startPoint, normalStart);
    var planeStart = new Cesium.Plane(normalStart, -Cesium.Cartesian3.distance(this.startPoint, new Cesium.Cartesian3(0, 0, 0)));
    var hypPoint = {};
    polyline.height = Cesium.Plane.getPointDistance(planeStart, endPoint);
    var labelPositonHeight = {};
    var labelPositonH = {};
    var labelPositonS = {};

    if (polyline.height <= 0) {
      Cesium.Plane.projectPointOntoPlane(planeStart, endPoint, hypPoint);
      Cesium.Cartesian3.midpoint(endPoint, hypPoint, labelPositonHeight);
      Cesium.Cartesian3.midpoint(this.startPoint, hypPoint, labelPositonH);
      polyline.distanceH = Cesium.Cartesian3.distance(this.startPoint, hypPoint);
    } else {
      var normalEnd = {};
      Cesium.Cartesian3.normalize(endPoint, normalEnd);
      var planeEnd = new Cesium.Plane(normalStart, -Cesium.Cartesian3.distance(endPoint, new Cesium.Cartesian3(0, 0, 0)));
      Cesium.Plane.projectPointOntoPlane(planeEnd, this.startPoint, hypPoint);
      Cesium.Cartesian3.midpoint(this.startPoint, hypPoint, labelPositonHeight);
      Cesium.Cartesian3.midpoint(endPoint, hypPoint, labelPositonH);
      polyline.distanceH = Cesium.Cartesian3.distance(endPoint, hypPoint);
    }

    polyline.distanceS = Cesium.Cartesian3.distance(this.startPoint, endPoint);
    Cesium.Cartesian3.midpoint(this.startPoint, endPoint, labelPositonS);
    polyline.height = Math.abs(polyline.height);

    if (polyline.positions.length !== 1) {
      polyline.positions.pop();
      polyline.positions.pop();
      labels.pop();
      labels.pop();
      labels.pop();
    }

    polyline.positions.push(endPoint);
    polyline.positions.push(hypPoint);
    var labelTextHeight = polyline.height > 1000 ? (polyline.height / 1000).toFixed(2) + 'km' : polyline.height.toFixed(2) + 'm';
    labels.push({
      text: this.$vc.lang.measure.verticalHeight + ': ' + labelTextHeight,
      position: labelPositonHeight
    });
    var labelTextH = polyline.distanceH > 1000 ? (polyline.distanceH / 1000).toFixed(2) + 'km' : polyline.distanceH.toFixed(2) + 'm';
    labels.push({
      text: this.$vc.lang.measure.horizontalDistance + ': ' + labelTextH,
      position: labelPositonH
    });
    var labelTextS = polyline.distanceS > 1000 ? (polyline.distanceS / 1000).toFixed(2) + 'km' : polyline.distanceS.toFixed(2) + 'm';
    labels.push({
      text: this.$vc.lang.measure.spaceDistance + ': ' + labelTextS,
      position: labelPositonS
    });
  },
  onMeasureEvt: function onMeasureEvt(polyline, index) {
    var flag = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    if (!this.depthTest) {
      var rs = Cesium.RenderState.fromCache({
        depthMask: true,
        depthTest: {
          enabled: false
        }
      });

      if (Cesium.SuperMapImageryProvider) {
        this.$refs.polylineCollection && (this.$refs.polylineCollection.cesiumObject._opaqueRS = rs);
      } else {
        this.$refs.polylineCollection && (this.$refs.polylineCollection.cesiumObject._opaqueRS.depthTest.enabled = false);
      }

      this.$refs.pointCollection && (this.$refs.pointCollection.cesiumObject._rsOpaque = rs);
      this.$refs.labelCollection.cesiumObject._billboardCollection._rsTranslucent = rs;
      this.$refs.labelCollection.cesiumObject._backgroundBillboardCollection._rsTranslucent = rs;
      var listener = this.$listeners.measureEvt;
      var type = this.type;

      if (type === 'distanceMeasuring' || type === 'areaMeasuring') {
        listener && this.$emit('measureEvt', {
          polyline: polyline,
          label: this.$refs.labelCollection.cesiumObject.get(index),
          type: type,
          finished: flag
        });
      } else {
        var labels = index;
        var labelsResult = {
          labelHeight: this.$refs.labelCollection.cesiumObject.get(labels.length - 3),
          labelH: this.$refs.labelCollection.cesiumObject.get(labels.length - 2),
          labelS: this.$refs.labelCollection.cesiumObject.get(labels.length - 1)
        };
        listener && this.$emit('measureEvt', {
          polyline: polyline,
          label: labelsResult,
          type: 'heightMeasuring',
          finished: flag
        });
      }
    }
  }
};
var mixinMeasure = {
  mixins: [cmp],
  props: props,
  watch: watch,
  methods: methods,
  created: function created() {
    var _this = this;

    _Object$defineProperties(this, {
      polyline: {
        enumerable: true,
        get: function get() {
          return _this.polyline;
        }
      }
    });
  },
  destroyed: function destroyed() {
    var handler = this.handler;

    if (handler) {
      handler.removeInputAction(Cesium.ScreenSpaceEventType.LEFT_CLICK);
      handler.removeInputAction(Cesium.ScreenSpaceEventType.MOUSE_MOVE);
      handler.removeInputAction(Cesium.ScreenSpaceEventType.RIGHT_CLICK);
      handler.destroy();
    }
  }
};

export default mixinMeasure;
